import sqlalchemy as sa
from sqlalchemy.dialects import mysql


def define_tables(meta, dialect=None):
    """
    Creates table definitions and adds them to schema catalogue.
    Use your application schema when integrating into your app for migrations
    support and other good things.

    :param meta: metadata catalogue to add to
    :param dialect: str, only required for mysql to switch payload to longtext
    :return: dict
    """
    tables = dict()

    # mysql needs longtext to store enough data in text column
    text_type = sa.Text if dialect != 'mysql' else mysql.LONGTEXT

    # events
    tables['events'] = sa.Table('event_store', meta,
        sa.Column('id', sa.Integer, primary_key=True, autoincrement=True),
        sa.Column('created', sa.DateTime, nullable=False, index=True),
        sa.Column('type', sa.String(256), nullable=False, index=True),
        sa.Column('author', sa.String(256), nullable=False, index=True),
        sa.Column('object_id', sa.String(256), nullable=False, index=True),
        sa.Column('payload', text_type, nullable=True),
        sa.Column('payload_rollback', text_type, nullable=True),
    )

    return tables




from shiftevent.handlers import BaseHandler


class Dummy3(BaseHandler):
    """
    Dummy handler 3
    This is mostly used for testing. The side effect it produces is modifying
    the event payload. Be careful, in reality you should never modify an event
    as they are immutable.
    """

    EVENT_TYPES = (
        'DUMMY_EVENT',
    )

    def handle(self, event):
        """ Handle event """
        payload = event.payload
        payload['dummy_handler3'] = 'processed'
        event.payload = payload
        return event

    def rollback(self, event):
        """ Rollback event """
        payload = event.payload
        if 'dummy_handler3' in payload:
            del payload['dummy_handler3']

        event.payload = payload
        return event





class EventException(Exception):
    """ Generic event exception marker """
    pass


class ConfigurationException(EventException, RuntimeError):
    """ Raised when configuration is invalid """
    pass


class DatabaseError(EventException, Exception):
    """ Generic database errors """
    pass


class EventError(DatabaseError, RuntimeError):
    """ Raised when there is an issue with event object """
    pass


class ProcessingUnsavedEvent(EventException, RuntimeError):
    """ Raised when emitting or rolling back  unsaved event"""
    pass


class MissingEventType(EventException, RuntimeError):
    """ Raised when handler implementation doesn't define EVENT_TYPE """
    pass


class UnsupportedEventType(EventException, RuntimeError):
    """ Raised when running a handler with unsupported event type """
    pass


class HandlerInstantiationError(EventException, RuntimeError):
    """ Raised when handlers are defined not as classes """
    pass


class InvalidEvent(EventException, RuntimeError):
    """ Raised when trying to persist invalid event """
    def __init__(self, *args, validation_errors=None, **kwargs):
        self.validation_errors = validation_errors
        super().__init__(*args, **kwargs)






#!/usr/bin/env python
import os
from setuptools import setup, find_packages

# ----------------------------------------------------------------------------
# Building
#
# Create source distribution:
# ./setup.py sdist
#
#
# Create binary distribution (non-univeral, python 3 only):
# ./setup.py bdist_wheel --python-tag=py3
#
# Register on PyPI:
# twine register dist/mypkg.whl
#
#
# Upload to PyPI:
# twine upload dist/*
#
# ----------------------------------------------------------------------------

# project version
version = '0.0.11'

# development status
# dev_status = '1 - Planning'
# dev_status = '2 - Pre-Alpha'
dev_status = '3 - Alpha'
# dev_status = '4 - Beta'
# dev_status = '5 - Production/Stable'
# dev_status = '6 - Mature'
# dev_status = '7 - Inactive'

# github repository url
repo = 'https://github.com/projectshift/shift-event'
license_type = 'MIT License'

# monkey patch os for vagrant hardlinks
del os.link

# run setup
setup(**dict(

    # author
    author='Dmitry Belyakov',
    author_email='dmitrybelyakov@gmail.com',

    # project meta
    name='shiftevent',
    version=version,
    url=repo,
    download_url=repo + '/archive/' + version + '.tar.gz',
    description='Simple event store for event-sourced systems',
    keywords=[
        'python3',
        'event sourcing',
    ],

    # classifiers
    # see: https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[

        # maturity
        'Development Status :: ' + dev_status,

        # license
        'License :: OSI Approved :: ' + license_type,

        # audience
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',

        # pythons
        'Programming Language :: Python :: 3',

        # categories
        'Topic :: Utilities',
        'Topic :: Software Development :: Libraries'
    ],

    # project packages
    packages=find_packages(exclude=['tests*']),

    # include none-code data files from manifest.in (http://goo.gl/Uf0Yxc)
    include_package_data=True,

    # project dependencies
    install_requires=[
        'shiftschema>=0.2.0,<0.3.0',
        'SQLAlchemy==1.2.7'
    ],


    # project license
    license=license_type
))



from shiftevent.handlers import BaseHandler


class NoTypes(BaseHandler):
    """
    This handler does not define EVENT_TYPES. It should always fail to
    instantiate. Used for testing
    """

    def handle(self, event):
        """ Handle event """
        pass

    def rollback(self, event):
        """ Rollback event """
        pass


